/*
=====================================================================

 sgftoadobe                         VERSION:  1.20  13 October, 1990  

 Chuck Ammon, ammon@seismic.geosc.psu.edu

=====================================================================
 PURPOSE:  To convert SAC Graphics Files 
		to an Adobe Illustrator Postscript File.
=====================================================================
 MODULE/LEVEL:  sgfcodes/1
=====================================================================
 MODIFICATION HISTORY:
    901013:  Added the color operator and changed text (Ammon)
	     Added the sizing option found at WCT's request
	     (op code = -8 )
    900327:  Added moveToFlag to eliminate unnecessary moves (Ammon)
    900311:  Added linestyles and line thicknesses options (Ammon)
    891114:  Revamped to put out Adobe Illustrator PostScript
              (Ammon - Penn State)
    890326:  Changed to include EPSF-like header and output positions
		in points (limit 300 dpi)	(Ammon - Penn State)
    880601:  Changed linestyles to line widths (Ammon and Mackay)
    870921:  Added optional linewidth and plot id label plus bug fixes
    870502:  Made it more robust and more readable
    870113:  Modified input arguments and some logic.
    860000:  Original version due to Tom Owens at Univ. of Missouri.

  GENERAL COMMENTS:
   This is not all my code, as is obvious from above, however,
    significant changes have been made from the original
    sgftops.  For readibility, I usually name all global
    variables beginning with a lower case g and 
    define'd variables with a 'k', functions begin with upper
    case letters. Some of these conventions may be ignored by
    older parts of the code.

    requires the math library (-lm) and the file adobe1.20.h

    !!!!!!! change the path name for adobe1.20.h ( 1 instance )

    compile: cc -o sgftoadobe sgftoadobe.c -lm

    The units of the plot are converted from sgf units to points,
    where 1 point = 1/72 inches.

=====================================================================
*/
#include <stdio.h>
#include <sys/param.h>
#include <math.h>

#define kMaxbuflen 5004
#define kX_OFFSET 106 /* used in function ToPoints */
#define kY_OFFSET 306

char *gLineThicknesses[5] = {"0.5 w","1.5 w","2.5 w","3.5 w","4.0 w"};
char *gLineDashes[5] = {"[]0 d","[1 3]0 d","[1 2 4 2]4 d","[3]0 d","[4 2]0 d"};
char *gColorOp[8] = {"1 1 1 0 K",
		     "0 1 1 0 K",
		     "1 0 1 0 K",
		     "1 0 0 0.5 K",
		     "0 0 1 0 K",
		     "1 0 0 0 K",
		     "0 1 0 0 K",
		     "0 0 0 1 K"};
/*  white red green blue yellow cyan magenta black */

char gLineType = 't'; /* t = thickness (default), d = dashes */

int gIVec=kMaxbuflen; /* global count for path length, then stroke */

/*******************************************************************

global variables for coordinate conversions 

*******************************************************************/

float gXfactor = 0.0135; /* 6" by 4.5" view space*/
float gYfactor = 0.0135;
   /* gXfactor =  0.0225  for 10" by 7.5" view space*/

#define minDist 0.024  /*   1/3000 of an inch = 0.024 points */
   /*minDist 0.48   1/150 of an inch = 0.24 points */

float gOldX = -1000, gOldY = -1000;

int gToF;
float gXmax = 0, gXmin = 1000;
float gYmax = 0, gYmin = 1000;

/* end of coordinate globals */

/*******************************************************************

		MAIN

*******************************************************************/
main(argc,argv)
int argc;
char *argv[];
{
  short buffer[kMaxbuflen];
  int buflen, i, width, done = 0;
  FILE *fp, *fopen(), *ofp;
  char *p1, *p2;
  char includeShowpage = 'n';
  short GetShort();
  char *dattim, usnam[L_cuserid+2], path[MAXPATHLEN];
  char *ctime();
  long rawtim;

	/* be sure we have enough arguments */
	if (argc < 3 || argc > 5 ) {
		AboutSGFtoAdobe();
		exit(-1);
	}

	/* does argv[1] end in ".sgf" ? */
	if (strcmp(&argv[1][strlen(argv[1])-4],".sgf") != 0) {
		fprintf(stderr,"%s: %s not a SAC GRAPHICS FILE.\n",*argv,argv[1]);
		exit(1);
	}

	/* open the sgf file */
	fp = fopen(argv[1],"r");
	if (fp == NULL) {
		fprintf(stderr,"%s: Can't open %s for reading.\n"
		,*argv,argv[1]);
		exit(-3);
	}

	/* open the postscript file to write */
	if ((ofp = fopen(argv[2],"w")) == NULL) {
		fprintf(stderr,"cannot open postscript file to write: '%s'\n",argv[2]);
		exit(-2);
	}

/*******************************************************************

	Get the file information for EPSF comments

*/
	rawtim = time(0);
	dattim = ctime(&rawtim);
	cuserid(usnam);
	getwd(path);

	fprintf(ofp,"%%!PS-Adobe-2.0 EPSF-1.2\n");
	fprintf(ofp,"%%%%Creator: Adobe Illustrator 88(TM) 1.8.3\n");
	fprintf(ofp,"%%%%For: (%s)\n",usnam);
	fprintf(ofp,"%%%%Title: (%s/%s)\n",path,argv[1]);
	fprintf(ofp,"%%%%CreationDate:  (%.24s)\n",dattim);

/*******************************************************************

	Copy the adobe prolog and set initialize the
	graphics state

*/
	CopyProlog(ofp);
	Initialize(ofp);

/*******************************************************************

	set the linestyle and showpage flags
*/

  if ( argc > 3 ) {
	  p1 = argv[3];
	  gLineType = *p1;
  } 
  if(argc > 4)
  {
     p2 = argv[4];
     includeShowpage = *p2;
  }


/*******************************************************************

	Loop through the sgf file

*/

/* execute this loop once for each input record */
  while (!done) {
    buflen = GetShort(fp);
    if ( buflen < 0 )
	 break;
    buflen = (buflen << 16) + GetShort(fp);
    buflen *= 2;
/* Buflen is now the number of short integers (not including the 4 bytes
   comprising buflen) in the next record.  */

    for (i = 0; i < buflen; i++) buffer[i] = GetShort(fp);
    HandleBuffer(buffer,buflen,&done,ofp);
  }
  (void) fclose(fp);

/*******************************************************************

	Finish up with the trailing postscript

*/

   if(includeShowpage == 'y')
      fprintf(ofp,"\nS\nshowpage");
   else
      fprintf(ofp,"\nS");

   fprintf(ofp,"\n%%%%Trailer\n");
   fprintf(ofp,"Adobe_Illustrator881 /terminate get exec\n");
   fprintf(ofp,"Adobe_customcolor /terminate get exec\n");
   fprintf(ofp,"Adobe_cshow /terminate get exec\n");
   fprintf(ofp,"Adobe_cmykcolor /terminate get exec\n");

  (void) fclose(ofp);
  exit(0);
}

/*******************************************************************/

HandleBuffer(buffer,buflen,done,ofp)
short *buffer;
int buflen;
int *done;
FILE *ofp;
{
  int i; /* gIVec is global now, so short buffers are ok */
  int x,y,count,dummy,moveToFlag,colorIndex,colorFlag;
  float fx,fy,moveToX,moveToY;
  char str[133];

  moveToFlag = 0;

  for (i = 0; i < buflen && !*done; ) {
    gIVec--;
    if (gIVec == 0) {
      moveToX = fx;
      moveToY = fy;
      moveToFlag = 1;
      gIVec = kMaxbuflen;
    }
    if (buffer[i] >= 0) {
      x = buffer[i++];
      y = buffer[i++];
      ToPoints(x,y,&fx,&fy,&gToF);
      if(gToF)
      {

	if(moveToFlag)
	{
	    if(colorFlag)
	     {
               fprintf(ofp,"\nS\n%s",gColorOp[colorIndex]);
	       fprintf(ofp,"\n%.2f %.2f m",moveToX,moveToY);
	       colorFlag = 0;
              }
	      else
	      {
	       fprintf(ofp,"\nS\n%.2f %.2f m",moveToX,moveToY);
	      }

	  moveToFlag = 0;
	}

      	fprintf(ofp,"\n%.2f %.2f l",fx,fy);
      }
    }
    else switch(buffer[i++]) {
      case -1 : break;
      case -2 : *done = 1;
		count = buffer[i++];
		break;
      case -3 : count = buffer[i++];
		x = buffer[i++];
		y = buffer[i++];
		ToPoints(x,y,&fx,&fy,&gToF);
		if(gToF)
		{
			/*fprintf(ofp,"\nS\n%.2f %.2f m",fx,fy);*/
			moveToX = fx;
			moveToY = fy;
			moveToFlag = 1;
	        }
		break;
      case -4 : count = buffer[i++]; /* the color operator */
		colorIndex = buffer[i++];
		colorFlag = 1;
		break;
      case -5 : count = buffer[i++];
		y = buffer[i++];
		for (x = 0; x < y; x += 2) {
		  str[x] = ((char *)buffer)[2*i];
		  str[x+1] = ((char *)buffer)[2*i+1];
		  i++;
		}
	 	str[y] = '\0';
		/*fprintf(ofp,"\n(%s)show",str);*/
		break;
      case -6 : i += 3;
		break;
      case -7 : count = buffer[i++];
		fprintf(ofp,"\nS");
		if (buffer[i] <=0)
		    buffer[i] = 1;

		if(gLineType == 't')
		fprintf(ofp,"\n%s",gLineThicknesses[buffer[i++]-1]);
		else
		fprintf(ofp,"\n%s",gLineDashes[buffer[i++]-1]);

		break;
     case -8  : count = buffer[i++];
	        if(count == 1)
     	        {
                  gXfactor *= (buffer[i]/1000);
		  gYfactor *= (buffer[i++]/1000);
                }
		if(count == 2)
		{
		  gXfactor *= (buffer[i++]/1000);
		  gYfactor *= (buffer[i++]/1000);
                }
		break;
      default : count = buffer[i++];
		i += count;
    }
  }
}

/*	GetShort - read a short integer from a stream
*/
short GetShort(fp)
FILE *fp;
{
  int hi;

  hi =	getc(fp);
  return((hi << 8) + getc(fp));
}


ToPoints(x,y,fx,fy,gToF)
int x,y,*gToF; 
float *fx,*fy;
{
	/*convert to points and check to make sure
	  that the point moves at least one unit
	  of plotter resolution.  If it doesn't,
	  don't bother plotting the point.	*/

   float distance;  /* the distance between the present position
			and the previous position in points */


	/* convert from sgf units to points */
	*fx = (float) x * gXfactor + kX_OFFSET;
	*fy = (float) y * gYfactor + kY_OFFSET;

	distance = (*fx - gOldX)*(*fx - gOldX) + (*fy - gOldY)*(*fy - gOldY);
	distance = (float) sqrt((double)distance);
	if(distance >= minDist)
		*gToF = 1;
	else
		*gToF = 0;

	if(x == 14 && y == 496)
	{			/* avoid pen move back to origin */
		*gToF = 0;       /* after the plot is finished */
				/* for the BoundingBox calculation */

	}
	
	else
	{
	
	/* calculate the extrema for the BoundingBox */

	if(*fx >= gXmax)
		gXmax = *fx;
	if(*fy >= gYmax)
		gYmax = *fy;
	if(*fx <= gXmin)
		gXmin = *fx;
	if(*fy <= gYmin)
		gYmin = *fy;
	}

	/* Store the current position for the next
		distance calculation	*/

	gOldX = *fx;
	gOldY = *fy;
}


CopyProlog(ofp)
FILE *ofp;
{
 FILE *fd;
 int c;
	     

   if((fd = fopen("/pubusr/local/bin/adobe1.20.h","r")) == (FILE *) NULL)
   {
      printf("\nCan't open Prolog File adobe1.20.h\n");
      exit(-1);
    }

   else
   {
	rewind(fd);
	while( (c = getc(fd)) != EOF)
	putc(c,ofp);
    }
}

Initialize(ofp)
FILE *ofp;
{
    fprintf(ofp,"\n0.5 w");
}

AboutSGFtoAdobe()
{
  fprintf(stderr,"\n\nUSAGE:\n");
  fprintf(stderr,"\nsgftoadobe sgf_file ps_file line_type include_showpage\n");
  fprintf(stderr,"\nwhere line_type is t or d for thickness or dash\n");
  fprintf(stderr,"and include_showpage is y or n (see below)\n");
  fprintf(stderr,"\nInclude a showpage if you are going to send the\n");
  fprintf(stderr,"  the file directly to a laser-printer.\n");
  fprintf(stderr,"Don't include a showpage if you are going to read\n");
  fprintf(stderr,"  the file into illustrator\n");
  fprintf(stderr,"\nDEFAULTS: line_type = t, include_showpage = n\n\n");

}
/* eof */

