      program sgftohipen
*=====================================================================
* PURPOSE:  To convert the contents of a SAC Graphics File (SGF) to a
*           file containing Houston Instruments pen plotter commands.
*=====================================================================
* MODULE/LEVEL:  sgf/1
*=====================================================================
* GLOBAL INPUT:
*    mach:    mcmsg, mcpfn, mcpw, munout
*=====================================================================
* SUBROUTINES CALLED:
*    local:   plotfile
*    saclib:  zgimsg, zgpmsg, indexb, poptok, modcase,
*=====================================================================
* MODIFICATION HISTORY:
*    870930:  Major restructuring.
*    851017:  Original version based on a program by Norm Burr.
*=====================================================================
* DOCUMENTED/REVIEWED:  870930
*=====================================================================

      include 'mach'

      character*(mcmsg) message, prompt
      character*(mcpw) token
      character*(mcpfn) kifile, kofile

      parameter (mhelp=4)
      character*(80) khelp(mhelp)
      data khelp/
     # 'This program converts the contents of a SGF',
     # 'to a file containing graphics commands for the',
     # 'Houston Instruments flat bed pen plotter.',
     # 'To use enter the name of a SGF and an output file.'/

* PROCEDURE:

* - Get message from input line or prompt user.

      call zgimsg(message)
      nc=indexb(message)
  100 if(nc.le.0)then
        do 200 j=1,mhelp
        nc=indexb(khelp(j))
  200   write(munout,'(a)')khelp(j)(1:nc)
        write(munout,'(1x)')
        prompt='Enter name of SGF and name of output file: $'
        call zgpmsg(prompt,message)
      endif

* - Decode input.
*   This is the name of the input SGF and the output file.

      ic=0
      nc=indexb(message)

      call poptok(message,nc,ic,ic1,ic2,itype)
      kifile=message(ic1:ic2)

      call poptok(message,nc,ic,ic1,ic2,itype)
      kofile=message(ic1:ic2)

* - Open output file.

C      call zgtfun(nofun,nerr)
      if(nerr.ne.0)go to 9000
      call znfile(nofun,kofile,'TEXT',ntused,nerr)
      if(nerr.ne.0)go to 9000

* - Write device initialization commands to the output file.
*   The first set of commands selects the pen plotter, homes the pen,
*   requests absolute addressing, sets the increment to 0.005,
*   and then goes into local mode. This allows the user a chance
*   to put pens into the holder and load a new sheet of paper.
*   The second set of commands selects pen 1 and sets default text
*   attributes, including rotating the text through 90 degrees.

      write(nofun,'('';:H A EC5 EL'')')
      write(nofun,'(''P1 S(S  5,NI,G0,X0,Y100)_'')')

* - Process input SGF.

      call plotfile(kifile,nofun,nerr)
      if(nerr.ne.0)go to 8888
        
* - Write device termination instructions to the output file.
*   This set of commands homes the pen, empties the pen holder,
*   and deselects the pen plotter.

      write(nofun,'(a)')'H P0 @'

* - Close output file and terminate program.

 8888 call zclose(nofun,ncerr)
      call zexit

* - Handle exceptions.

 9000 nc=indexb(kofile)
      write(munout,'(2a)')'ERROR: opening output file ',kofile(1:nc)
      go to 8888

      end

      subroutine plotfile(kifile,nofun,nerr)
*=====================================================================
* PURPOSE:  To plot the contents of a SGF.
*=====================================================================
* INPUT ARGUMENTS:
*    kifile:  name of input SGF to merge. [c]
*    nofun:   fortran file unit to write to. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    nerr:    error return flag. [i]
*=====================================================================
* MODULE/LEVEL:  sgf/2
*=====================================================================
* GLOBAL INPUT:
*    mach:    munout
*=====================================================================
* SUBROUTINES CALLED:
*    local:   plotbuffer
*    saclib:  zopen, zrabs, zclose
*=====================================================================
* MODIFICATION HISTORY:
*    870115:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870115
*=====================================================================

      include 'mach'
      character*(*) kifile
      parameter (mbuf=5200)
      integer*2 buf(mbuf)

      nerr=0

* - Open input SGF.

C      call zgtfun(nifun,nerr)
      call zopen(nifun,kifile,'RODATA',nlen,nerr)
      if(nerr.ne.0)go to 9000

* - Loop through input SGF, one buffer at a time.

      ilcdsk=0
      ibuf=0
      ldone=.false.
 1000 if(.not.ldone)then

* -- Read in number of long (32-bit) words in next buffer.
        call zrabs(nifun,ilnbuf,1,ilcdsk,nerr)
        if (nerr.ne.0) go to 9100
        ilcdsk=ilcdsk+1

* -- Read in buffer.
        call zrabs(nifun,buf,ilnbuf,ilcdsk,nerr)
        if (nerr.ne.0) go to 9100
        ilcdsk=ilcdsk+ilnbuf
        ibuf=ibuf+1

* -- Scale contents of buffer.
        nlnbuf=2*ilnbuf
        call plotbuffer(buf,nlnbuf,nofun,ldone)

* -- Loop until done.
        go to 1000
      endif

* - Close input SGF.

      call zclose(nifun,ncerr)

 8888 return

 9000 nc=indexb(kifile)
      write(munout,'(2a)')'ERROR: opening file ',kifile(1:nc)
      go to 8888

 9100 nc=indexb(kifile)
      write(munout,'(2a)')'ERROR: reading file ',kifile(1:nc)
      call zclose(nifun,ncerr)
      go to 8888

      end

      subroutine plotbuffer(buf,nlnbuf,nofun,ldone)
*=====================================================================
* PURPOSE:  To plot a SGF buffer.
*=====================================================================
* INPUT ARGUMENTS:
*    buf:     buffer of graphics commands. [si]
*    nlnbuf:  (short integer) length of buf. [i]
*    nofun:   output fortran file unit. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    ldone:   set to .true. when end-of-frame command encounted. [l]
*=====================================================================
* MODULE/LEVEL:  sgf/3
*=====================================================================
* GLOBAL INPUT:
*    mach:    mcmsg
*=====================================================================
* MODIFICATION HISTORY:
*    870115:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870115
*=====================================================================

      include 'mach'
      integer*2 buf
      dimension buf(*)
      character*(mcmsg) ktext

      parameter (ixmaxpen=1500, iymaxpen=2000, ifacpen=16)
      parameter (maxpen=8)
      parameter (maxline=10)

      data icurpen/1/

* PROCEDURE:

* - Initialize some variables.

      jbuf=1
      ldone=.false.
      nbadloc=0

* - Loop on each command in buffer:

 1000 if(jbuf.le.nlnbuf)then
        icom=buf(jbuf)

* -- Draw command.

        if(icom.ge.0)then
* --- Send pen change if necessary.
          if(ipen.ne.icurpen)then
            write(nofun,'(a,i1)')'P',ipen
            icurpen=ipen
          endif
* --- Validate location.
*     "ixloc" is a long integer location in the range 0 to 32000
*     "iyloc" is a long integer location in the range 0 to 24000
          call validateloc2(buf(jbuf),buf(jbuf+1),ixloc,iyloc,nbadloc)
* --- Convert to pen plotter coordinates.  Note that the x and y
*     axes are flipped and the x axis is reversed.
          iypen=ixloc/ifacpen
          ixpen=ixmaxpen-iyloc/ifacpen
* --- Write draw opcode.
          write(nofun,'(a,i5,a,i5)')'D',ixpen,',',iypen
          jbuf=jbuf+2

* -- Move command

        elseif(icom.eq.-3)then
          call validateloc2(buf(jbuf+2),buf(jbuf+3),ixloc,iyloc,nbadloc)
          iypen=ixloc/ifacpen
          ixpen=ixmaxpen-iyloc/ifacpen
          write(nofun,'(a,i5,a,i5)')'U',ixpen,',',iypen
          jbuf=jbuf+2+buf(jbuf+1)

* -- Color command.
*     "icolor" is a positive integer from 1 to the maximum number
*     of colors in the color table.
*    (This is interpreted as a pen change for this program.)

        elseif(icom.eq.-4)then
          ipen=buf(jbuf+2)
          ipen=iabs(mod(ipen,maxpen))+1
          jbuf=jbuf+2+buf(jbuf+1)

* -- Hardware text command.
*     "text" is the character string.
*     "nctext" is the character count.

        elseif(icom.eq.-5)then
          nctext=buf(jbuf+2)
          call zgetc(buf(jbuf+3),ktext,nctext)
          write(nofun,'(3a)')'S(S  5)',ktext(1:nctext),'_'
          jbuf=jbuf+2+buf(jbuf+1)

* -- Hardware text size command.

        elseif(icom.eq.-6)then
          isize=buf(jbuf+3)/(ifacpen*9)
          write(nofun,'(a,i3,a)')'S(S',isize,')'
          jbuf=jbuf+2+buf(jbuf+1)

* -- Linestyle command.
*     "iline" is positive integer.
*     A value of 1 is a solid line.  Other values are device dependent.

        elseif(icom.eq.-7)then
          iline=buf(jbuf+2)
          iline=iabs(mod(iline-1,maxline))+1
          write(nofun,'(a,i1)')'L',iline
          jbuf=jbuf+2+buf(jbuf+1)

* -- Plot size command.
*     "xsize" is requested horizontal size of plot in inches.
*    (This command is ignored by this program.)

        elseif(icom.eq.-8)then
          xsize=0.001*float(buf(jbuf+2))
          jbuf=jbuf+2+buf(jbuf+1)

* -- No-op command.  Used to fill a buffer to a 32-bit word boundary.

        elseif(icom.eq.-1)then
          jbuf=jbuf+1

* -- End-of-plot command.

        elseif(icom.eq.-2)then
          ldone=.true.
          jbuf=jbuf+2
          go to 8888

* -- Skip unknown opcodes and continue.

        else
          jbuf=jbuf+2+buf(jbuf+1)

        endif

* - Loop until buffer is empty.
        go to 1000

      endif

 8888 return

      end

      subroutine validateloc2(ixin,iyin,ixout,iyout,nbadloc)
*=====================================================================
* PURPOSE:  To validate a SAC graphics file location.
*           Also converts location from short integers to long integers.
*=====================================================================
* INPUT ARGUMENTS:
*    ixin:    sgf x location from file. [si]
*    iyin:    sgf y location from file. [si]
*=====================================================================
* OUTPUT ARGUMENTS:
*    ixout:   sgf x location after validation. [i]
*    iyout:   sgf y location after validation. [i]
*    nbadloc: incremented by 1 if the input location was bad. [i]
*=====================================================================
* MODULE/LEVEL:  sgf/4
*=====================================================================
* GLOBAL INPUT:
*    mach:
*=====================================================================
* MODIFICATION HISTORY:
*    870929:  Made maximum x and y locations into parameters.
*    851017:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  851017
*=====================================================================

      include 'mach'
      parameter (ixmax=32000, iymax=24000)

      integer*2 ixin, iyin
      integer*4 ixout, iyout

* PROCEDURE:

      ixout=ixin
      if(ixout .lt. 0)then
        ixout=0
        nbadloc=nbadloc+1
      elseif(ixout .gt. ixmax)then
        ixout=ixmax
        nbadloc=nbadloc+1
      endif

      iyout=iyin
      if(iyout .lt. 0)then
        yout=0
        nbadloc=nbadloc+1
      elseif(iyout .gt. iymax)then
        iyout=iymax
        nbadloc=nbadloc+1
      endif

 8888 return

      end
