/*
*=====================================================================
* PURPOSE:  To convert SAC Graphics Files to a MIF (Maker Interchange) File.
*=====================================================================
* MODULE/LEVEL:  sgfcodes/1
*=====================================================================
* DEPENDENT ROUTINES:
*   sgftoeps:  Produces MIF for FrameMaker.
*=====================================================================
* MODIFICATION HISTORY:
* NOTE: Additions and deletions to this program may affect sgftoeps.
*    920912:  Original version.
*=====================================================================
*/
#include <stdio.h>
#include <sys/param.h>

#define TRUE 1
#define FALSE 0
#define maxbuflen 5004
#define NUMLSTYLS 10                              /* 10 line styles */
#define NUMLWIDTHS 10                             /* 10 line widths */

#define SCALE_FACTOR 0.00025

static char scale[]="yes";            /* for prompt or command line parameter */
static int scale_flag = TRUE;
static int mx,my, npts;
static int iangle;

static int groupID1, groupID2;
static int igroup;


int ivec=400; /* global count for path length, then stroke */

/*----------------------------------------------------------------------------*/
main(argc,argv)
     int argc;
     char *argv[];
{
  short buffer[maxbuflen];
  int fp, buflen, i, width, done = 0, ict;
  FILE  *fopen(), *ofp, *fpc;
  int penw;

  npts = 0;
  iangle = 0;
  groupID1 = FALSE;
  groupID2 = FALSE;
  igroup = 2;

  /* be sure we have enough arguments */
  if (argc < 3 || argc > 6 ) {
    fprintf(stderr,"usage: %s sgf_file ps_file",*argv);
    fprintf(stderr," [ line_width id_flag scale_flag]\n");
    fprintf(stderr," where:  line_width = 1, 2, or 3\n");
    fprintf(stderr,"         scale_flag = yes or no\n");
    fprintf(stderr,"  example: sgftomif foo.sgf foo.ps 2 yes no\n");
    fprintf(stderr,"      for line width 2, no scaling\n");
    exit(-1);
  }

  /* does argv[1] end in ".sgf" ? */
  if ( strcmp(&argv[1][strlen(argv[1])-4],".sgf") ) {
    fprintf(stderr,"%s: %s not a SAC GRAPHICS FILE.\n",*argv,argv[1]);
    exit(1);
  }

  /* open the sgf file */
  fp = open(argv[1],0);
  if (fp == -1) {
    fprintf(stderr,"%s: Can't open %s for reading.\n",*argv,argv[1]);
    exit(-3);
  }

  /* open the mif file to write */
  if ((ofp = fopen(argv[2],"w")) == NULL) {
    fprintf(stderr,"cannot open mif file to write: '%s'\n",argv[2]);
    exit(-2);
  }

  /* Write the initialization junk to file */
  fprintf(ofp,"<MIFFile 3.00>#Generated by sgftomif; identifies this as a MIF file>");

  if ( argc > 3 ) {
    sscanf(argv[3],"%d",&penw);
    fprintf(ofp,"\n<PenWidth %f>",penw*0.1);
  } 

  /* add the optional label for userid, dir, file, date and time */
  if ( argc > 4 && !strncmp(argv[4],"yes",3) ) {
  }

  /* set the scale flag on or off */
  if ( argc > 5 && !strncmp(argv[5],"no",2) )
    scale_flag = FALSE;

  /* execute this loop once for each input record */
  while (!done) {
    if ( read(fp,&buflen,4) != 4 )
      break;

    buflen *= 2;
    
    /* 
      Buflen is now the number of short integers (not including the 4 bytes
      comprising buflen) in the next record.  
      NOTE: read uses byte count and execute_buffer use a count of short
      integers, hence the repeated *2 and /2 operations, but this
      will take care of any truncated files.
    */

    if ( (buflen = read(fp,buffer,2*buflen)) < 0 )
      break;

    execute_buffer(buffer,buflen/2,&done,ofp,ict);
  } /* while */

  close(fp);

  /* end the frame */
  for (i=1; i<=igroup; i++)
      fprintf(ofp,"<Group <ID %d>>",i);
  (void) fclose(ofp);
  exit(0);
}

/* -----------------------------------------------------------------------*/
execute_buffer(buffer,buflen,done,ofp,ict)
     short *buffer;
     int buflen;
     int *done;
     FILE *ofp;
{
  int i,indx; /* ivec is global now, so short buffers are ok */
  int x,y,count,dx,dy;
  char str[133];

  float width, height;                     /* width and height of viewspace */
                                           /* for hardware text size */

  float xsize;                             /* requested horizontal size in " */
  float xscale, yscale;
  float sx, sy;
	
  int igray, fillFlag;
  
  for (i = 0; i < buflen && !*done; ) {
    ivec--;
    if (ivec == 0) {
      ivec = 400;
    }
    if (buffer[i] >= 0) {
      x = buffer[i++];
      y = buffer[i++];
      if (npts == 0)
	  {
	  scale_xy(mx,my,&sx,&sy);
	  fprintf(ofp,"\n<PolyLine <GroupID %d><Point %f %f>",igroup,sx,sy);
	  }
      npts++;
      scale_xy(x,y,&sx,&sy);
      fprintf(ofp,"\n<Point %f %f>", sx,sy);
    }
    else 
      {
      if (npts > 0)
	    {
	    EndPath(ofp,&fillFlag);
	    npts = 0;
	    }
      switch(buffer[i++]) {
      case -1 : break;                             /* No-op command */
      case -2 : *done = 1;                         /* End of plot command */
	count = buffer[i++];
	break;
      case -3 : count = buffer[i++];               /* Move command */
	mx = buffer[i++];
	my = buffer[i++];
	break;
      case -4 : count = buffer[i++];               /* Color command - default SAC color map  */
	if (buffer[i] <0)
	  buffer[i] = 0;                
	if (buffer[i] == 1)
	  buffer[i] = 0;                
	fprintf(ofp,"\n<Separation %d>",(buffer[i++])%7);
	break;
      case -5 : count = buffer[i++];               /* Hardware text command */
        i += 5;   /* modified due to change in hardware text spec. by Ammon 10-3-94 */
	y = buffer[i++];
	for (x = 0; x < y; x += 2) {
	  str[x] = ((char *)buffer)[2*i];
	  str[x+1] = ((char *)buffer)[2*i+1];
	  i++;
	}
	str[y] = '\0';
	scale_xy(mx,my,&sx,&sy);
	fprintf(ofp,"\n<TextLine <GroupID 1><TLOrigin %f %f><Angle %d><String %s>>",sx,sy,iangle,str);
	groupID2 = TRUE;
	break;
      case -6 : count = buffer[i++];               /* Hardware text size */
	width = (float)buffer[i++]*SCALE_FACTOR;
	height =(float)buffer[i++]*SCALE_FACTOR;
	break;
      case -7 : count = buffer[i++];               /* Linestyle command */
	if (buffer[i] <0)
	  buffer[i] = 0;                
	fprintf(ofp,"\n<Pen %d>",(buffer[i++]-1)%NUMLSTYLS);
	break;
      case -8 : count=buffer[i++];                 /* Plot size command */
	i++;
	break;
      case -9 : count = buffer[i++];               /* Line-width command */
	fprintf(ofp,"\n<PenWidth %f>",buffer[i++]*0.1);
	break;
      case -10:				/* Polyfill Command */
	count = buffer[i++];
	igray = buffer[i++];
	count = buffer[i++];	/* Move command */
	count = buffer[i++];
	x = buffer[i++];
	y = buffer[i++];
	scale_xy(x,y,&sx,&sy);
	fprintf(ofp,"\n<Fill %d><Polygon <GroupID %><Point %f %f>",igray,igroup,sx,sy);
	fillFlag = 1;
        npts++;
	break;
      case -12 : count = buffer[i++];               /* Text angle command */
	iangle = buffer[i++];
	if ((45<=iangle) && (iangle<135))
	    iangle = 90;
	else if ((135<=iangle) && (iangle<225))
	    iangle = 180;
	else if ((225<=iangle) && (iangle<315))
	    iangle = 270;
	else
	    iangle = 0;
	break;
      default : count = buffer[i++];               /* Skip unknown op-code */
        printf("unknown op code: %i",count);
	i += count;
      }
    }
  }
}


/***************************************************************************/
scale_xy(mx,my,sx,sy)
    int		mx, my;
    float	*sx, *sy;
{
    *sy = 24000 - my;
    *sy = *sy * SCALE_FACTOR;
    *sx = mx * SCALE_FACTOR;
}

/***************************************************************************/

EndPath(ofp, fillFlag)
	int             *fillFlag;
	FILE           *ofp;
{
	groupID1 = TRUE;
	fprintf(ofp, ">");
	if (*fillFlag == 1) {
		fprintf(ofp, "<Fill 15>");
		*fillFlag = 0;
	}
}


/* eof */
