      program sgftoquic
*=====================================================================
* PURPOSE:  To convert the contents of a SAC Graphics File (SGF) to a
*           file containing QUIC commands used by QMS laserprinters.
*=====================================================================
* MODULE/LEVEL:  sgf/1
*=====================================================================
* GLOBAL INPUT:
*    mach:    mcmsg, mcpfn, mcpw, munout
*=====================================================================
* SUBROUTINES CALLED:
*    local:   plotfile
*    saclib:  zgimsg, zgpmsg, indexb, poptok, modcase,
*=====================================================================
* MODIFICATION HISTORY:
*    900312:  Changed scaling factor for plot size opcode.
*    870930:  Major restructuring.
*    851017:  Original version based on a program by Norm Burr.
*=====================================================================
* DOCUMENTED/REVIEWED:  870930
*=====================================================================

      include 'mach'

      character*(mcmsg) message, prompt
      character*(mcpw) token
      character*(mcpfn) kifile, kofile

      parameter (mhelp=3)
      character*(80) khelp(mhelp)
      data khelp/
     # 'This program converts the contents of a SGF to a file',
     # 'containing QUIC commands for the QMS laserprinter.',
     # 'To use enter the name of a SGF and an output file.'/

* PROCEDURE:

* - Get message from input line or prompt user.

      call zgimsg(message)
      nc=indexb(message)
  100 if(nc.le.0)then
        do 200 j=1,mhelp
        nc=indexb(khelp(j))
  200   write(munout,'(a)')khelp(j)(1:nc)
        write(munout,'(1x)')
        prompt='Enter name of SGF and name of output file: $'
        call zgpmsg(prompt,message)
      endif

* - Decode input.
*   This is the name of the input SGF and the output file.

      ic=0
      nc=indexb(message)

      call poptok(message,nc,ic,ic1,ic2,itype)
      kifile=message(ic1:ic2)

      call poptok(message,nc,ic,ic1,ic2,itype)
      kofile=message(ic1:ic2)

* - Open output file.

C      call zgtfun(nofun,nerr)
      if(nerr.ne.0)go to 9000
      call znfile(nofun,kofile,'TEXT',ntused,nerr)
      if(nerr.ne.0)go to 9000

* - Write device initialization commands to the output file.
*   The first line puts the laser printer into QUIC command mode.
*   The second line sets up landscape (rather than portrait) mode.
*   The third line puts the printer into vector mode, sets the linestyle
*   to solid, linewidth to 5 pixels, and selects a default hardware font.

      write(nofun,'(a)')'^py^-'
      write(nofun,'(a)')'^iol'
      write(nofun,'(a)')'^igv^v0^pw05^is204'

* - Process input SGF.

      call plotfile(kifile,nofun,nerr)
      if(nerr.ne.0)go to 8888
        
* - Write device initialization commands to the output file.
*   The first line gets the printer out of vector mode.
*   The second line sets up portrait mode.
*   The third line turns QUIC command mode off.

      write(nofun,'(a)')'^ige^-'
      write(nofun,'(a)')'^ion'
      write(nofun,'(a)')'^pn^-'

* - Close output file and terminate program.

 8888 call zclose(nofun,ncerr)
      call zexit

* - Handle exceptions.

 9000 nc=indexb(kofile)
      write(munout,'(2a)')'ERROR: opening output file ',kofile(1:nc)
      go to 8888

      end

      subroutine plotfile(kifile,nofun,nerr)
*=====================================================================
* PURPOSE:  To plot the contents of a SGF.
*=====================================================================
* INPUT ARGUMENTS:
*    kifile:  name of input SGF to merge. [c]
*    nofun:   fortran file unit to write to. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    nerr:    error return flag. [i]
*=====================================================================
* MODULE/LEVEL:  sgf/2
*=====================================================================
* GLOBAL INPUT:
*    mach:    munout
*=====================================================================
* SUBROUTINES CALLED:
*    local:   plotbuffer
*    saclib:  zopen, zrabs, zclose
*=====================================================================
* MODIFICATION HISTORY:
*    870115:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870115
*=====================================================================

      include 'mach'
      character*(*) kifile
      parameter (mbuf=5200)
      integer*2 buf(mbuf)

      nerr=0

* - Open input SGF.

C      call zgtfun(nifun,nerr)
      call zopen(nifun,kifile,'RODATA',nlen,nerr)
      if(nerr.ne.0)go to 9000

* - Loop through input SGF, one buffer at a time.

      ilcdsk=0
      ibuf=0
      ldone=.false.
 1000 if(.not.ldone)then

* -- Read in number of long (32-bit) words in next buffer.
        call zrabs(nifun,ilnbuf,1,ilcdsk,nerr)
        if (nerr.ne.0) go to 9100
        ilcdsk=ilcdsk+1

* -- Read in buffer.
        call zrabs(nifun,buf,ilnbuf,ilcdsk,nerr)
        if (nerr.ne.0) go to 9100
        ilcdsk=ilcdsk+ilnbuf
        ibuf=ibuf+1

* -- Scale contents of buffer.
        nlnbuf=2*ilnbuf
        call plotbuffer(buf,nlnbuf,nofun,ldone)

* -- Loop until done.
        go to 1000
      endif

* - Close input SGF.

      call zclose(nifun,ncerr)

 8888 return

 9000 nc=indexb(kifile)
      write(munout,'(2a)')'ERROR: opening file ',kifile(1:nc)
      go to 8888

 9100 nc=indexb(kifile)
      write(munout,'(2a)')'ERROR: reading file ',kifile(1:nc)
      call zclose(nifun,ncerr)
      go to 8888

      end

      subroutine plotbuffer(buf,nlnbuf,nofun,ldone)
*=====================================================================
* PURPOSE:  To plot a SGF buffer.
*=====================================================================
* INPUT ARGUMENTS:
*    buf:     buffer of graphics commands. [si]
*    nlnbuf:  (short integer) length of buf. [i]
*    nofun:   output fortran file unit. [i]
*=====================================================================
* OUTPUT ARGUMENTS:
*    ldone:   set to .true. when end-of-frame command encounted. [l]
*=====================================================================
* MODULE/LEVEL:  sgf/3
*=====================================================================
* GLOBAL INPUT:
*    mach:    mcmsg
*=====================================================================
* MODIFICATION HISTORY:
*    870115:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  870115
*=====================================================================

      include 'mach'
      integer*2 buf
      dimension buf(*)
      character*(mcmsg) ktext

      parameter (mwrite=80)
      character*(mwrite) kwrite 
      parameter (ixmaxpen=1500, iymaxpen=2000, ifacpen=16)
      parameter (maxline=8)
      character*(1) kline(maxline)

      save kwrite
      data kline/'0','2','3','4','6','7','8','9'/
      data scale/0.32000/,ixmin/0138/,iymax/7897/
      data nc/1/

* PROCEDURE:

* - Initialize some variables.

      jbuf=1
      ldone=.false.
      nbadloc=0

* - Loop on each command in buffer:

 1000 if(jbuf.le.nlnbuf)then
        icom=buf(jbuf)

* -- Draw command.

        if(icom.ge.0)then
* --- Validate location.
*     "xloc" is a real location in the range 0. to 32000.
*     "yloc" is a real location in the range 0. to 24000.
          call validateloc1(buf(jbuf),buf(jbuf+1),xloc,yloc,nbadloc)
* --- Convert to QUIC coordinates.  Note that the y axes is reversed.
          iyloc=ixmin+ifix(xloc*scale)
          iyloc=iymax-ifix(yloc*scale)
* --- Add  draw opcode to character string.
          write(kwrite(nc:nc+12),'(a,i5,a,i5)')'^d',ixpen,':',iypen
          nc=nc+13
* --- Write string if almost full.
          if((nc+13).gt.mwrite)then
            write(nofun,'(a)')kwrite(1:nc-1)
            nc=1
          endif
          jbuf=jbuf+2

* -- Move command

        elseif(icom.eq.-3)then
          call validateloc1(buf(jbuf+2),buf(jbuf+3),xloc,yloc,nbadloc)
          iyloc=ixmin+ifix(xloc*scale)
          iyloc=iymax-ifix(yloc*scale)
          write(kwrite(nc:nc+12),'(a,i5,a,i5)')'^u',ixpen,':',iypen
          nc=nc+13
          if((nc+13).gt.mwrite)then
            write(nofun,'(a)')kwrite(1:nc-1)
            nc=1
          endif
          jbuf=jbuf+2+buf(jbuf+1)

* -- Color command.
*     "icolor" is a positive integer from 1 to the maximum number
*     of colors in the color table.
*    (This command is ignored by this program.)

        elseif(icom.eq.-4)then
          icolor=buf(jbuf+2)
          jbuf=jbuf+2+buf(jbuf+1)

* -- Hardware text command.
*     "text" is the character string.
*     "nctext" is the character count.

        elseif(icom.eq.-5)then
          nctext=buf(jbuf+2)
          call zgetc(buf(jbuf+3),ktext,nctext)
          if(nc.gt.1)then
            write(nofun,'(a)')kwrite(1:nc-1)
            nc=1
          endif
          write(nofun,'(a,i5.5,a,i5.5,1x,2a)')
     #         '^ige^t',ix,'^j',iy,ktext(1:nctext),'^igv^-'
          jbuf=jbuf+2+buf(jbuf+1)

* -- Hardware text size command.
*    (This command is ignored by this program.)

        elseif(icom.eq.-6)then
          jbuf=jbuf+2+buf(jbuf+1)

* -- Linestyle command.
*     "iline" is positive integer.
*     A value of 1 is a solid line.  Other values are device dependent.

        elseif(icom.eq.-7)then
          iline=buf(jbuf+2)
          iline=iabs(mod(iline-1,maxline))+1
          if(nc.gt.1)then
            write(nofun,'(a)')kwrite(1:nc-1)
            nc=1
          endif
          write(nofun,'(2a)')'^v',kline(iline)
          jbuf=jbuf+2+buf(jbuf+1)

* -- Plot size command.
*     "xsize" is requested horizontal size of plot in inches.
*    (This command is ignored by this program.)

        elseif(icom.eq.-8)then
          xsize=0.01*float(buf(jbuf+2))
          jbuf=jbuf+2+buf(jbuf+1)

* -- No-op command.  Used to fill a buffer to a 32-bit word boundary.

        elseif(icom.eq.-1)then
          jbuf=jbuf+1

* -- End-of-plot command.

        elseif(icom.eq.-2)then
          if(nc.gt.1)then
            write(nofun,'(a)')kwrite(1:nc-1)
            nc=1
          endif
          ldone=.true.
          jbuf=jbuf+2
          go to 8888

* -- Skip unknown opcodes and continue.

        else
          jbuf=jbuf+2+buf(jbuf+1)

        endif

* - Loop until buffer is empty.
        go to 1000

      endif

 8888 return

      end

      subroutine validateloc1(ixin,iyin,xout,yout,nbadloc)
*=====================================================================
* PURPOSE:  To validate a SAC graphics file location.
*           Also converts location from short integers to reals.
*=====================================================================
* INPUT ARGUMENTS:
*    ixin:    sgf x location from file. [si]
*    iyin:    sgf y location from file. [si]
*=====================================================================
* OUTPUT ARGUMENTS:
*    xout:    sgf x location after validation. [f]
*    yout:    sgf y location after validation. [f]
*    nbadloc: incremented by 1 if the input location was bad. [i]
*=====================================================================
* MODULE/LEVEL:  sgf/4
*=====================================================================
* GLOBAL INPUT:
*    mach:
*=====================================================================
* MODIFICATION HISTORY:
*    870929:  Made maximum x and y locations into parameters.
*    851017:  Original version.
*=====================================================================
* DOCUMENTED/REVIEWED:  851017
*=====================================================================

      include 'mach'
      parameter (xmax=32000., ymax=24000.)

      integer*2 ixin, iyin
      real*4 xout, yout

* PROCEDURE:

      xout=ixin
      if(xout .lt. 0.0)then
        xout=0.0
        nbadloc=nbadloc+1
      elseif(xout .gt. xmax)then
        xout=xmax
        nbadloc=nbadloc+1
      endif

      yout=iyin
      if(yout .lt. 0.0)then
        yout=0.0
        nbadloc=nbadloc+1
      elseif(yout .gt. ymax)then
        yout=ymax
        nbadloc=nbadloc+1
      endif

 8888 return

      end
